/* infolister.js
 * Main file.
 * Requires: common.js
 *
 * Part of InfoLister by Nickolay Ponomarev <asqueella@gmail.com>
 * Copyright (c) 2004-2005, Nickolay Ponomarev.
 * See the license.txt included in this package for licensing information.
 */

var InfoLister = {
  get mode() {
    return ILPrefs.getIntPref("ouputmode");
  },
  get html() {
    return this.mode == 1;
  },

/** overlay window **/
  onLoad: function()
  { // called on main application load (actually, for each new window)
    try {
      var filename = ILPrefs.getFilePref("file");
      InfoLister.saveToFile(filename);
    } catch(e) {} // invalid file or no file set
  },

/** IL window event handlers - xxx will be eventually moved outside InfoLister obj **/
  uploadInfo: function() {
    if(ILPrefs.getUnicharPrefDef("ftp.host", "") == "" ||
       ILPrefs.getUnicharPrefDef("ftp.path", "") == "") {
      var ps = ILHelpers.getService("embedcomp/prompt-service;1", "nsIPromptService");
      // xxx l10n
      var titleText = "InfoLister";
      var msgText = "You have to set FTP options before you can upload.\n\n" +
        "Do you want to set them now?";
      var setupNowText = "Setup now";
      var rv = ps.confirmEx(window, titleText, msgText,
        ps.BUTTON_TITLE_IS_STRING * ps.BUTTON_POS_0 +
        ps.BUTTON_TITLE_CANCEL * ps.BUTTON_POS_1, setupNowText, null, null,
        "", {value:false});
        
      if(rv == 0) { // setup now
        var prefsWindow = InfoListerWindows.open(InfoListerWindows.Prefs);
        // xxx focus the ftp tab
      }
      return;
    }
    var f = this.makeLocalFile();
    InfoLister.saveToFile(f);
    InfoListerWindows.startUpload(f.path); // xxx pass nsIFile here instead
  },
  
  printInfo: function(aEvent, aData)
  { // puts the text generated by generateInfo() to a textbox on the form
    document.getElementById("info").selectedIndex = (this.mode == 1) ? 1:0;
    if(!aData)
      aData = this.generateInfo();

    if(this.html) {
      var doc = document.getElementById("infoFrame").contentDocument;
      var body = doc.getElementsByTagName("body")[0];
      body.innerHTML = aData;
    } else {
      var infoPad = document.getElementById("infoPad");
      infoPad.value = aData;
    }
  },
  
  pickAndSave: function()
  { // (used in onclick button handler). Selects a file, and saves the info
    // generated by generateInfo() in it.
    var rv = ILHelpers.pickFile("", null);
    if(!rv) return;
    
    var defaultMode = this.mode;
    if(defaultMode != rv.mode)
	  ILPrefs.setIntPref("ouputmode", rv.mode);
    var data = this.formatter.fixLineBreaks(this.generateInfo());
    if(defaultMode != rv.mode)
	  ILPrefs.setIntPref("ouputmode", defaultMode);

    this.saveToFile(rv.file, data); 
  },


  // generates information about your GRE application, returns it as a string
  generateInfo: function()
  {
    var Modes = ["Text.", "HTML.", "XML."];
    var prefs = ILPrefs.getSubBranch(Modes[this.mode]);
    // create an XML tree with all needed information
    var data = ILInfoCollector.collectData(prefs);

    // and format it
    return this.formatter.formatInfo(data, prefs);
  },
  
  onFrameClick: function(e) {
    if(e.target.localName.toLowerCase() == "a") {
      e.preventDefault();
      var url = e.target.href;
      if(url)
        this.openLink(url);
    }
  },

  openLink: function(aURL) {
    if(ILHelpers.hostApp == ILHelpers.AppGuids.Firefox) {
      var w = InfoListerWindows.open(InfoListerWindows.Browser);
      w.gBrowser.selectedTab = w.gBrowser.addTab(aURL);
    } else {
      var ioService = ILHelpers.getService("network/io-service;1", "nsIIOService");
      var uri = ioService.newURI(aURL, null, null);
      var extProtocolSvc = ILHelpers.getService(
        "uriloader/external-protocol-service;1", "nsIExternalProtocolService");
      extProtocolSvc.loadUrl(uri);
    }
  },
  
  
  /**********   file I/O stuff   **********************************************/
  getFileByPath: function(aPath) {
    var file = ILHelpers.createInstance("file/local;1", "nsILocalFile");
    file.initWithPath(aPath);
    return file;
  },
				
  saveToFile: function(aFile, aData) {
    try {
      ILHelpers.QI(aFile, "nsILocalFile");
    } catch(e) {
      aFile = InfoLister.getFileByPath(aFile);
    }
    
    ILHelpers.QI(aFile, "nsILocalFile");
    if(!aData)
      aData = InfoLister.generateInfo();

    var stream = ILHelpers.createInstance("network/file-output-stream;1",
      "nsIFileOutputStream");
    stream.init(aFile, 0x20|0x02|0x08, 0666, 0);

    try {
      var charset = ILPrefs.getLocalizedUnicharPref("charset");
    } catch(e) {}
    try {
      var uniConv = ILHelpers.getService("intl/scriptableunicodeconverter",
        "nsIScriptableUnicodeConverter");
      uniConv.charset = charset ? charset : "UTF-8";
      aData = uniConv.ConvertFromUnicode(aData);
    } catch(e) {
      ILHelpers.dumpException(e);
    }

    stream.write(aData, aData.length);
    stream.close();
  },
  
  // return the nsILocalFile to save info to
  makeLocalFile: function() {
    try {
      return ILPrefs.getFilePref("file");
    } catch(e) {
      // if no filename specified, generate a file in user_profile/chrome
      var file = ILHelpers.getService("file/directory_service;1",
              "nsIProperties").get("UChrm", Components.interfaces.nsILocalFile);
      file.append("infolister." + this.fileExtension);
      return file;
    }
  },
  
  get fileExtension() {
    var extensions = ["txt", "html", "xml"];
    return extensions[this.mode];
  }
};

InfoLister.formatter = new ILFormatter();

